@@ -13,18 +13,10 @@
 #include <stdlib.h>
 #include "itclInt.h"
 
-static Tcl_ObjCmdProc ItclFinishCmd;
+static Tcl_NamespaceDeleteProc FreeItclObjectInfo;
 static Tcl_ObjCmdProc ItclSetHullWindowName;
 static Tcl_ObjCmdProc ItclCheckSetItclHull;
 
-#ifdef OBJ_REF_COUNT_DEBUG
-static Tcl_ObjCmdProc ItclDumpRefCountInfo;
-#endif
-
-#ifdef ITCL_PRESERVE_DEBUG
-static Tcl_ObjCmdProc ItclDumpPreserveInfo;
-#endif
-
 MODULE_SCOPE const ItclStubs itclStubs;
 
 static int Initialize(Tcl_Interp *interp);
@@ -103,39 +95,29 @@ static const char *clazzClassScript =
 
 #define ITCL_IS_ENSEMBLE 0x1
 
-typedef struct ItclCmdsInfo {
-    const char *name;
-    int flags;
-} ItclCmdsInfo;
-static ItclCmdsInfo itclCmds [] = {
-    { "::itcl::class", 0},
-    { "::itcl::find", ITCL_IS_ENSEMBLE},
-    { "::itcl::delete", ITCL_IS_ENSEMBLE},
-    { "::itcl::is", ITCL_IS_ENSEMBLE},
-    { "::itcl::filter", ITCL_IS_ENSEMBLE},
-    { "::itcl::forward", ITCL_IS_ENSEMBLE},
-    { "::itcl::import::stub", ITCL_IS_ENSEMBLE},
-    { "::itcl::mixin", ITCL_IS_ENSEMBLE},
-    { "::itcl::parser::delegate", ITCL_IS_ENSEMBLE},
-    { "::itcl::type", 0},
-    { "::itcl::widget", 0},
-    { "::itcl::widgetadaptor", 0},
-    { "::itcl::nwidget", 0},
-    { "::itcl::addoption", 0},
-    { "::itcl::addobjectoption", 0},
-    { "::itcl::adddelegatedoption", 0},
-    { "::itcl::adddelegatedmethod", 0},
-    { "::itcl::addcomponent", 0},
-    { "::itcl::setcomponent", 0},
-    { "::itcl::extendedclass", 0},
-    { "::itcl::genericclass", 0},
-    { "::itcl::parser::delegate", ITCL_IS_ENSEMBLE},
-    { NULL, 0},
-};
 #ifdef ITCL_DEBUG_C_INTERFACE
 extern void RegisterDebugCFunctions( Tcl_Interp * interp);
 #endif
 
+static Tcl_ObjectMetadataDeleteProc Demolition;
+
+static const Tcl_ObjectMetadataType canary = {
+    TCL_OO_METADATA_VERSION_CURRENT,
+    "Itcl Foundations",
+    Demolition,
+    NULL
+};
+
+void
+Demolition(
+    void *clientData)
+{
+    ItclObjectInfo *infoPtr = (ItclObjectInfo *)clientData;
+
+    infoPtr->clazzObjectPtr = NULL;
+    infoPtr->clazzClassPtr = NULL;
+}
+
 static const Tcl_ObjectMetadataType objMDT = {
     TCL_OO_METADATA_VERSION_CURRENT,
     "ItclObject",
@@ -155,37 +137,19 @@ const Tcl_MethodType itclRootMethodType = {
 
 static int
 RootCallProc(
-    ClientData clientData,
+    void *clientData,
     Tcl_Interp *interp,
     Tcl_ObjectContext context,
     int objc,
     Tcl_Obj *const *objv)
 {
     Tcl_Object oPtr = Tcl_ObjectContextObject(context);
-    ItclObject *ioPtr = Tcl_ObjectGetMetadata(oPtr, &objMDT);
+    ItclObject *ioPtr = (ItclObject *)Tcl_ObjectGetMetadata(oPtr, &objMDT);
     ItclRootMethodProc *proc = (ItclRootMethodProc *)clientData;
 
     return (*proc)(ioPtr, interp, objc, objv);
 }
 
-/*
- * ------------------------------------------------------------------------
- *  FreeItclObjectInfo()
- *
- *  called when an interp is deleted to free up memory
- *
- * ------------------------------------------------------------------------
- */
-static void
-FreeItclObjectInfo(
-    ClientData clientData)
-{
-    ItclObjectInfo *infoPtr;
-
-    infoPtr = (ItclObjectInfo *)clientData;
-    ItclFinishCmd(infoPtr, infoPtr->interp, 0, NULL);
-}
-
 /*
  * ------------------------------------------------------------------------
  *  Initialize()
@@ -196,10 +160,6 @@ FreeItclObjectInfo(
  * ------------------------------------------------------------------------
  */
 
-#ifdef NEW_PROTO_RESOLVER
-int ItclVarsAndCommandResolveInit(Tcl_Interp *interp);
-#endif
-
 static int
 Initialize (
     Tcl_Interp *interp)
@@ -212,8 +172,9 @@ Initialize (
     char *res_option;
     int opt;
     int isNew;
+    Tcl_Class tclCls;
     Tcl_Object clazzObjectPtr, root;
-    Tcl_Obj *objPtr;
+    Tcl_Obj *objPtr, *resPtr;
 
     if (Tcl_InitStubs(interp, "8.6", 0) == NULL) {
         return TCL_ERROR;
@@ -224,42 +185,36 @@ Initialize (
         return TCL_ERROR;
     }
 
-    nsPtr = Tcl_CreateNamespace(interp, ITCL_NAMESPACE, NULL, NULL);
+    objPtr = Tcl_NewStringObj("::oo::class", -1);
+    Tcl_IncrRefCount(objPtr);
+    clazzObjectPtr = Tcl_GetObjectFromObj(interp, objPtr);
+    if (!clazzObjectPtr || !(tclCls = Tcl_GetObjectAsClass(clazzObjectPtr))) {
+	Tcl_DecrRefCount(objPtr);
+        return TCL_ERROR;
+    }
+    Tcl_DecrRefCount(objPtr);
+
+    infoPtr = (ItclObjectInfo*)Itcl_Alloc(sizeof(ItclObjectInfo));
+
+    nsPtr = Tcl_CreateNamespace(interp, ITCL_NAMESPACE, infoPtr, FreeItclObjectInfo);
     if (nsPtr == NULL) {
+	Itcl_Free(infoPtr);
         Tcl_Panic("Itcl: cannot create namespace: \"%s\" \n", ITCL_NAMESPACE);
     }
 
-    nsPtr = Tcl_CreateNamespace(interp, ITCL_NAMESPACE"::internal::dicts",
+    nsPtr = Tcl_CreateNamespace(interp, ITCL_INTDICTS_NAMESPACE,
             NULL, NULL);
     if (nsPtr == NULL) {
+	Itcl_Free(infoPtr);
         Tcl_Panic("Itcl: cannot create namespace: \"%s::internal::dicts\" \n",
 	        ITCL_NAMESPACE);
     }
 
-    Tcl_CreateObjCommand(interp, ITCL_NAMESPACE"::finish", ItclFinishCmd,
-            NULL, NULL);
-
-    /* for debugging only !!! */
-#ifdef OBJ_REF_COUNT_DEBUG
-    Tcl_CreateObjCommand(interp,
-            ITCL_NAMESPACE"::dumprefcountinfo",
-            ItclDumpRefCountInfo, NULL, NULL);
-#endif
-
-#ifdef ITCL_PRESERVE_DEBUG
-    Tcl_CreateObjCommand(interp,
-            ITCL_NAMESPACE"::dumppreserveinfo",
-            ItclDumpPreserveInfo, NULL, NULL);
-#endif
-    /* END for debugging only !!! */
-
     /*
      *  Create the top-level data structure for tracking objects.
      *  Store this as "associated data" for easy access, but link
      *  it to the itcl namespace for ownership.
      */
-    infoPtr = (ItclObjectInfo*)ckalloc(sizeof(ItclObjectInfo));
-    memset(infoPtr, 0, sizeof(ItclObjectInfo));
     infoPtr->interp = interp;
     infoPtr->class_meta_type = (Tcl_ObjectMetadataType *)ckalloc(
             sizeof(Tcl_ObjectMetadataType));
@@ -279,6 +234,7 @@ Initialize (
     Tcl_InitHashTable(&infoPtr->instances, TCL_STRING_KEYS);
     Tcl_InitHashTable(&infoPtr->frameContext, TCL_ONE_WORD_KEYS);
     Tcl_InitObjHashTable(&infoPtr->classTypes);
+
     infoPtr->ensembleInfo = (EnsembleInfo *)ckalloc(sizeof(EnsembleInfo));
     memset(infoPtr->ensembleInfo, 0, sizeof(EnsembleInfo));
     Tcl_InitHashTable(&infoPtr->ensembleInfo->ensembles, TCL_ONE_WORD_KEYS);
@@ -291,19 +247,19 @@ Initialize (
     Tcl_IncrRefCount(infoPtr->typeDestructorArgumentPtr);
     infoPtr->lastIoPtr = NULL;
 
-    Tcl_SetVar(interp, ITCL_NAMESPACE"::internal::dicts::classes", "", 0);
-    Tcl_SetVar(interp, ITCL_NAMESPACE"::internal::dicts::objects", "", 0);
-    Tcl_SetVar(interp, ITCL_NAMESPACE"::internal::dicts::classOptions", "", 0);
-    Tcl_SetVar(interp,
-            ITCL_NAMESPACE"::internal::dicts::classDelegatedOptions", "", 0);
-    Tcl_SetVar(interp,
-            ITCL_NAMESPACE"::internal::dicts::classComponents", "", 0);
-    Tcl_SetVar(interp,
-            ITCL_NAMESPACE"::internal::dicts::classVariables", "", 0);
-    Tcl_SetVar(interp,
-            ITCL_NAMESPACE"::internal::dicts::classFunctions", "", 0);
-    Tcl_SetVar(interp,
-            ITCL_NAMESPACE"::internal::dicts::classDelegatedFunctions", "", 0);
+    Tcl_SetVar2(interp, ITCL_NAMESPACE"::internal::dicts::classes", NULL, "", 0);
+    Tcl_SetVar2(interp, ITCL_NAMESPACE"::internal::dicts::objects", NULL, "", 0);
+    Tcl_SetVar2(interp, ITCL_NAMESPACE"::internal::dicts::classOptions", NULL, "", 0);
+    Tcl_SetVar2(interp,
+            ITCL_NAMESPACE"::internal::dicts::classDelegatedOptions", NULL, "", 0);
+    Tcl_SetVar2(interp,
+            ITCL_NAMESPACE"::internal::dicts::classComponents", NULL, "", 0);
+    Tcl_SetVar2(interp,
+            ITCL_NAMESPACE"::internal::dicts::classVariables", NULL, "", 0);
+    Tcl_SetVar2(interp,
+            ITCL_NAMESPACE"::internal::dicts::classFunctions", NULL, "", 0);
+    Tcl_SetVar2(interp,
+            ITCL_NAMESPACE"::internal::dicts::classDelegatedFunctions", NULL, "", 0);
 
     hPtr = Tcl_CreateHashEntry(&infoPtr->classTypes,
             (char *)Tcl_NewStringObj("class", -1), &isNew);
@@ -330,37 +286,35 @@ Initialize (
     infoPtr->useOldResolvers = opt;
     Itcl_InitStack(&infoPtr->clsStack);
 
-    Tcl_SetAssocData(interp, ITCL_INTERP_DATA,
-        (Tcl_InterpDeleteProc*)FreeItclObjectInfo, (ClientData)infoPtr);
+    Tcl_SetAssocData(interp, ITCL_INTERP_DATA, NULL, infoPtr);
 
-    Itcl_PreserveData((ClientData)infoPtr);
+    Itcl_PreserveData(infoPtr);
 
-#ifdef NEW_PROTO_RESOLVER
-    ItclVarsAndCommandResolveInit(interp);
-#endif
-
-    objPtr = Tcl_NewStringObj("::oo::class", -1);
-    root = Tcl_NewObjectInstance(interp, Tcl_GetObjectAsClass(
-	    Tcl_GetObjectFromObj(interp, objPtr)), "::itcl::Root",
+    root = Tcl_NewObjectInstance(interp, tclCls, "::itcl::Root",
 	    NULL, 0, NULL, 0);
-    Tcl_DecrRefCount(objPtr);
 
     Tcl_NewMethod(interp, Tcl_GetObjectAsClass(root),
 	    Tcl_NewStringObj("unknown", -1), 0, &itclRootMethodType,
-	    ItclUnknownGuts);
+	    (void *)ItclUnknownGuts);
     Tcl_NewMethod(interp, Tcl_GetObjectAsClass(root),
 	    Tcl_NewStringObj("ItclConstructBase", -1), 0,
-	    &itclRootMethodType, ItclConstructGuts);
+	    &itclRootMethodType, (void *)ItclConstructGuts);
     Tcl_NewMethod(interp, Tcl_GetObjectAsClass(root),
 	    Tcl_NewStringObj("info", -1), 1,
-	    &itclRootMethodType, ItclInfoGuts);
+	    &itclRootMethodType, (void *)ItclInfoGuts);
 
     /* first create the Itcl base class as root of itcl classes */
     if (Tcl_EvalEx(interp, clazzClassScript, -1, 0) != TCL_OK) {
         Tcl_Panic("cannot create Itcl root class ::itcl::clazz");
     }
-    clazzObjectPtr = Tcl_GetObjectFromObj(interp, Tcl_GetObjResult(interp));
-
+    resPtr = Tcl_GetObjResult(interp);
+    /*
+     * Tcl_GetObjectFromObject can call Tcl_SetObjResult, so increment the
+     * refcount first.
+     */
+    Tcl_IncrRefCount(resPtr);
+    clazzObjectPtr = Tcl_GetObjectFromObj(interp, resPtr);
+    Tcl_DecrRefCount(resPtr);
 
     if (clazzObjectPtr == NULL) {
         Tcl_AppendResult(interp,
@@ -369,11 +323,9 @@ Initialize (
         return TCL_ERROR;
     }
 
-    /* work around for SF bug #254 needed because of problem in TclOO 1.0.2 !! */
-    if (Tcl_PkgPresent(interp, "TclOO", "1.0.2", 1) != NULL) {
-	Itcl_IncrObjectRefCount(clazzObjectPtr);
-    }
+    Tcl_ObjectSetMetadata(clazzObjectPtr, &canary, infoPtr);
 
+    infoPtr->clazzObjectPtr = clazzObjectPtr;
     infoPtr->clazzClassPtr = Tcl_GetObjectAsClass(clazzObjectPtr);
 
     /*
@@ -399,7 +351,7 @@ Initialize (
      *  Export all commands in the "itcl" namespace so that they
      *  can be imported with something like "namespace import itcl::*"
      */
-    itclNs = Tcl_FindNamespace(interp, "::itcl", (Tcl_Namespace*)NULL,
+    itclNs = Tcl_FindNamespace(interp, "::itcl", NULL,
         TCL_LEAVE_ERR_MSG);
 
     /*
@@ -436,14 +388,14 @@ Initialize (
      *  Set up the variables containing version info.
      */
 
-    Tcl_SetVar(interp, "::itcl::version", ITCL_VERSION, TCL_NAMESPACE_ONLY);
-    Tcl_SetVar(interp, "::itcl::patchLevel", ITCL_PATCH_LEVEL,
+    Tcl_SetVar2(interp, "::itcl::version", NULL, ITCL_VERSION, TCL_NAMESPACE_ONLY);
+    Tcl_SetVar2(interp, "::itcl::patchLevel", NULL, ITCL_PATCH_LEVEL,
             TCL_NAMESPACE_ONLY);
 
 
 #ifdef ITCL_DEBUG_C_INTERFACE
     RegisterDebugCFunctions(interp);
-#endif    
+#endif
     /*
      *  Package is now loaded.
      */
@@ -513,16 +465,17 @@ Itcl_SafeInit (
  */
 static int
 ItclSetHullWindowName(
-    ClientData clientData,   /* infoPtr */
-    Tcl_Interp *interp,      /* current interpreter */
+    void *clientData,   /* infoPtr */
+    Tcl_Interp *dummy,      /* current interpreter */
     int objc,                /* number of arguments */
     Tcl_Obj *const objv[])   /* argument objects */
 {
     ItclObjectInfo *infoPtr;
+    (void)dummy;
 
     infoPtr = (ItclObjectInfo *)clientData;
-    if (infoPtr->currIoPtr != NULL) {
-        infoPtr->currIoPtr->hullWindowNamePtr = objv[1];
+    if ((infoPtr->currIoPtr != NULL) && (objc > 1)) {
+	infoPtr->currIoPtr->hullWindowNamePtr = objv[1];
 	Tcl_IncrRefCount(infoPtr->currIoPtr->hullWindowNamePtr);
     }
     return TCL_OK;
@@ -537,7 +490,7 @@ ItclSetHullWindowName(
  */
 static int
 ItclCheckSetItclHull(
-    ClientData clientData,   /* infoPtr */
+    void *clientData,   /* infoPtr */
     Tcl_Interp *interp,      /* current interpreter */
     int objc,                /* number of arguments */
     Tcl_Obj *const objv[])   /* argument objects */
@@ -555,7 +508,7 @@ ItclCheckSetItclHull(
 	return TCL_ERROR;
     }
 
-    /* 
+    /*
      * This is an internal command, and is never called with an
      * objectName value other than the empty list. Check that with
      * an assertion so alternative handling can be removed.
@@ -578,7 +531,7 @@ ItclCheckSetItclHull(
 	        " variable for object \"", Tcl_GetString(objv[1]), "\"", NULL);
 	return TCL_ERROR;
     }
-    ivPtr = Tcl_GetHashValue(hPtr);
+    ivPtr = (ItclVariable *)Tcl_GetHashValue(hPtr);
     valueStr = Tcl_GetString(objv[2]);
     if (strcmp(valueStr, "2") == 0) {
         ivPtr->initted = 2;
@@ -596,243 +549,50 @@ ItclCheckSetItclHull(
 
 /*
  * ------------------------------------------------------------------------
- *  ItclFinishCmd()
+ *  FreeItclObjectInfo()
  *
- *  called when an interp is deleted to free up memory or called explicitly
- *  to check memory leaks
+ *  called when an interp is deleted to free up memory
  *
  * ------------------------------------------------------------------------
  */
-static int
-ItclFinishCmd(
-    ClientData clientData,   /* unused */
-    Tcl_Interp *interp,      /* current interpreter */
-    int objc,                /* number of arguments */
-    Tcl_Obj *const objv[])   /* argument objects */
+static void
+FreeItclObjectInfo(
+    void *clientData)
 {
-    Tcl_HashEntry *hPtr;
-    Tcl_HashSearch place;
-    Tcl_Namespace *nsPtr;
-    Tcl_Obj **newObjv;
-    Tcl_Obj *objPtr;
-    Tcl_Obj *ensObjPtr;
-    Tcl_Command cmdPtr;
-    Tcl_Obj *mapDict;
-    ItclObjectInfo *infoPtr;
-    ItclCmdsInfo *iciPtr;
-    int checkMemoryLeaks;
-    int i;
-    int result;
+    ItclObjectInfo *infoPtr = (ItclObjectInfo *)clientData;
 
-    ItclShowArgs(1, "ItclFinishCmd", objc, objv);
-    result = TCL_OK;
-    infoPtr = Tcl_GetAssocData(interp, ITCL_INTERP_DATA, NULL);
-    if (infoPtr == NULL) {
-        infoPtr = (ItclObjectInfo *)clientData;
-    }
-    checkMemoryLeaks = 0;
-    if (objc > 1) {
-        if (strcmp(Tcl_GetString(objv[1]), "checkmemoryleaks") == 0) {
-	    /* if we have that option, the namespace of the Tcl ensembles
-	     * is not teared down, so we have to simulate it here to
-	     * have the correct reference counts for infoPtr->infoVars2Ptr
-	     * infoPtr->infoVars3Ptr and infoPtr->infoVars4Ptr
-	     */
-	    checkMemoryLeaks = 1;
-	}
-    }
-    newObjv = (Tcl_Obj **)ckalloc(sizeof(Tcl_Obj *) * 2);
-    newObjv[0] = Tcl_NewStringObj("my", -1);;
-    for (i = 0; ;i++) {
-        iciPtr = &itclCmds[i];
-        if (iciPtr->name == NULL) {
-	    break;
-	}
-	if ((iciPtr->flags & ITCL_IS_ENSEMBLE) == 0) {
-            result = Itcl_RenameCommand(interp, iciPtr->name, "");
-	} else {
-	    objPtr = Tcl_NewStringObj(iciPtr->name, -1);
-            newObjv[1] = objPtr;
-	    Itcl_EnsembleDeleteCmd(infoPtr, infoPtr->interp, 2, newObjv);
-	    Tcl_DecrRefCount(objPtr);
-	}
-        iciPtr++;
-    }
-    Tcl_DecrRefCount(newObjv[0]);
-    ckfree((char *)newObjv);
-
-    /* remove the unknow handler, to free the reference to the
-     * Tcl_Obj with the name of it */
-    ensObjPtr = Tcl_NewStringObj("::itcl::builtin::Info::delegated", -1);
-    cmdPtr = Tcl_FindEnsemble(interp, ensObjPtr, TCL_LEAVE_ERR_MSG);
-    if (cmdPtr != NULL) {
-        Tcl_SetEnsembleUnknownHandler(NULL, cmdPtr, NULL);
-    }
-    Tcl_DecrRefCount(ensObjPtr);
-
-    while (1) {
-        hPtr = Tcl_FirstHashEntry(&infoPtr->instances, &place);
-	if (hPtr == NULL) {
-	    break;
-	}
-	Tcl_DeleteHashEntry(hPtr);
-    }
     Tcl_DeleteHashTable(&infoPtr->instances);
-
-    while (1) {
-        hPtr = Tcl_FirstHashEntry(&infoPtr->classTypes, &place);
-	if (hPtr == NULL) {
-	    break;
-	}
-	Tcl_DeleteHashEntry(hPtr);
-    }
     Tcl_DeleteHashTable(&infoPtr->classTypes);
-
     Tcl_DeleteHashTable(&infoPtr->procMethods);
-
     Tcl_DeleteHashTable(&infoPtr->objectCmds);
     Tcl_DeleteHashTable(&infoPtr->classes);
     Tcl_DeleteHashTable(&infoPtr->nameClasses);
     Tcl_DeleteHashTable(&infoPtr->namespaceClasses);
 
-    nsPtr = Tcl_FindNamespace(interp, "::itcl::parser", NULL, 0);
-    if (nsPtr != NULL) {
-        Tcl_DeleteNamespace(nsPtr);
-    }
+    assert (infoPtr->infoVarsPtr == NULL);
+    assert (infoPtr->infoVars4Ptr == NULL);
 
-    mapDict = NULL;
-    ensObjPtr = Tcl_NewStringObj("::itcl::builtin::Info", -1);
-    if (Tcl_FindNamespace(interp, Tcl_GetString(ensObjPtr), NULL, 0) != NULL) {
-        Tcl_SetEnsembleUnknownHandler(NULL,
-                Tcl_FindEnsemble(interp, ensObjPtr, TCL_LEAVE_ERR_MSG),
-	        NULL);
-    }
-    Tcl_DecrRefCount(ensObjPtr);
-
-    /* remove the vars entry from the info dict */
-    /* and replace it by the original one */
-    cmdPtr = Tcl_FindCommand(interp, "info", NULL, TCL_GLOBAL_ONLY);
-    if (cmdPtr != NULL && Tcl_IsEnsemble(cmdPtr)) {
-        Tcl_GetEnsembleMappingDict(NULL, cmdPtr, &mapDict);
-        if (mapDict != NULL) {
-
-            objPtr = Tcl_NewStringObj("vars", -1);
-	    Tcl_DictObjRemove(interp, mapDict, objPtr);
-	    Tcl_DictObjPut(interp, mapDict, objPtr, infoPtr->infoVars4Ptr);
-	    Tcl_DecrRefCount(objPtr);
-	    Tcl_SetEnsembleMappingDict(interp, cmdPtr, mapDict);
-        }
-    }
-    /* FIXME have to figure out why the refCount of
-     * ::itcl::builtin::Info
-     * and ::itcl::builtin::Info::vars and vars is 2 here !! */
-    /* seems to be as the tclOO commands are not yet deleted ?? */
-    Tcl_DecrRefCount(infoPtr->infoVars3Ptr);
-    Tcl_DecrRefCount(infoPtr->infoVars4Ptr);
-    if (checkMemoryLeaks) {
-        Tcl_DecrRefCount(infoPtr->infoVars3Ptr);
-        Tcl_DecrRefCount(infoPtr->infoVars4Ptr);
-    /* see comment above */
-    }
-
-    Tcl_DecrRefCount(infoPtr->typeDestructorArgumentPtr);
-
-    Tcl_EvalEx(infoPtr->interp,
-            "::oo::define ::itcl::clazz deletemethod unknown", -1, 0);
-
-    /* first have to look for the remaining memory leaks, then remove the next ifdef */
-    Itcl_RenameCommand(infoPtr->interp, "::itcl::clazz", "");
-
-    /* tear down ::itcl namespace (this includes ::itcl::parser namespace) */
-    nsPtr = Tcl_FindNamespace(infoPtr->interp, "::itcl::parser", NULL, 0);
-    if (nsPtr != NULL) {
-        Tcl_DeleteNamespace(nsPtr);
-    }
-    nsPtr = Tcl_FindNamespace(infoPtr->interp, "::itcl::import", NULL, 0);
-    if (nsPtr != NULL) {
-        Tcl_DeleteNamespace(nsPtr);
-    }
-    nsPtr = Tcl_FindNamespace(infoPtr->interp, "::itcl::internal", NULL, 0);
-    if (nsPtr != NULL) {
-        Tcl_DeleteNamespace(nsPtr);
-    }
-    nsPtr = Tcl_FindNamespace(infoPtr->interp, "::itcl::builtin", NULL, 0);
-    if (nsPtr != NULL) {
-        Tcl_DeleteNamespace(nsPtr);
-    }
-    nsPtr = Tcl_FindNamespace(infoPtr->interp, "::itcl", NULL, 0);
-    if (nsPtr != NULL) {
-        Tcl_DeleteNamespace(nsPtr);
+    if (infoPtr->typeDestructorArgumentPtr) {
+	Tcl_DecrRefCount(infoPtr->typeDestructorArgumentPtr);
+	infoPtr->typeDestructorArgumentPtr = NULL;
     }
 
     /* cleanup ensemble info */
-    ItclFinishEnsemble(infoPtr);
+    if (infoPtr->ensembleInfo) {
+	Tcl_DeleteHashTable(&infoPtr->ensembleInfo->ensembles);
+	Tcl_DeleteHashTable(&infoPtr->ensembleInfo->subEnsembles);
+	ItclFinishEnsemble(infoPtr);
+	ckfree((char *)infoPtr->ensembleInfo);
+	infoPtr->ensembleInfo = NULL;
+    }
 
-    ckfree((char *)infoPtr->class_meta_type);
+    if (infoPtr->class_meta_type) {
+	ckfree((char *)infoPtr->class_meta_type);
+	infoPtr->class_meta_type = NULL;
+    }
 
-    Itcl_DeleteStack(&infoPtr->clsStack);
     /* clean up list pool */
     Itcl_FinishList();
 
-    Itcl_ReleaseData((ClientData)infoPtr);
-    return result;
+    Itcl_ReleaseData(infoPtr);
 }
-
-#ifdef OBJ_REF_COUNT_DEBUG
-void Tcl_DbDumpRefCountInfo(const char *fileName, int noDeleted);
-
-
-/*
- * ------------------------------------------------------------------------
- *  ItclDumpRefCountInfo()
- *
- *  debugging routine to check for memory leaks in use of Tcl_Obj's
- *
- * ------------------------------------------------------------------------
- */
-static int
-ItclDumpRefCountInfo(
-    ClientData clientData,   /* unused */
-    Tcl_Interp *interp,      /* current interpreter */
-    int objc,                /* number of arguments */
-    Tcl_Obj *const objv[])   /* argument objects */
-{
-    int noDeleted;
-
-    noDeleted = 0;
-    if (objc > 1) {
-        if (strcmp(Tcl_GetString(objv[1]), "-nodeleted") == 0) {
-	    noDeleted = 1;
-	}
-    }
-    ItclShowArgs(0, "ItclDumpRefCountInfo", objc, objv);
-    Tcl_DbDumpRefCountInfo(NULL, noDeleted);
-    return TCL_OK;
-}
-#endif
-
-#ifdef ITCL_PRESERVE_DEBUG
-void Itcl_DbDumpPreserveInfo(const char *fileName);
-
-
-/*
- * ------------------------------------------------------------------------
- *  ItclDumpPreserveInfo()
- *
- *  debugging routine to check for memory leaks in use of Itcl_PreserveData
- *  and Itcl_ReleaseData
- *
- * ------------------------------------------------------------------------
- */
-static int
-ItclDumpPreserveInfo(
-    ClientData clientData,   /* unused */
-    Tcl_Interp *interp,      /* current interpreter */
-    int objc,                /* number of arguments */
-    Tcl_Obj *const objv[])   /* argument objects */
-{
-    ItclShowArgs(0, "ItclDumpPreserveInfo", objc, objv);
-    Itcl_DbDumpPreserveInfo(NULL);
-    return TCL_OK;
-}
-#endif
